#include <ch.h>
#include <hal.h>
#include <chprintf.h>

#include "board.h"
#include "usbcfg.h"

/**
 * blinking thread to show that we are alive
 */
static WORKING_AREA(blinkingThreadStack, 128);

static void blinkingThread(void *arg) {
	chRegSetThreadName("blinking thread");

	while (TRUE) {
		palSetPad(GPIOD, GPIOD_LED5); // red - blinks via thread
		chThdSleepMilliseconds(100);
		palClearPad(GPIOD, GPIOD_LED5); // red - blinks via thread
		chThdSleepMilliseconds(100);
	}
}

/* Virtual serial port over USB.*/
SerialUSBDriver SDU1;

void usb_serial_start(void) {
	/*
	 * Initializes a serial-over-USB CDC driver.
	 */
	sduObjectInit(&SDU1);
	sduStart(&SDU1, &serusbcfg);

	/*
	 * Activates the USB driver and then the USB bus pull-up on D+.
	 * Note, a delay is inserted in order to not have to disconnect the cable
	 * after a reset.
	 */
	usbDisconnectBus(serusbcfg.usbp);
	chThdSleepMilliseconds(1000);
	usbStart(serusbcfg.usbp, &usbcfg);
	usbConnectBus(serusbcfg.usbp);

	/*
	 * Activates the serial driver 2 using the driver default configuration.
	 * PA2(TX) and PA3(RX) are routed to USART2.
	 */
	sdStart(&SD2, NULL);
	palSetPadMode(GPIOA, 2, PAL_MODE_ALTERNATE(7));
	palSetPadMode(GPIOA, 3, PAL_MODE_ALTERNATE(7));

	while (TRUE) {
		if (SDU1.config->usbp->state == USB_ACTIVE) {
			break;
		}
		chThdSleepMilliseconds(500);
	}
}

#define TICKS_IN_MS  (CH_FREQUENCY / 1000)

#define PWM_SLOW	&PWMD4

static PWMConfig pwmcfg_slow = { 10000, 1000, NULL, { {
PWM_OUTPUT_DISABLED, NULL }, { PWM_OUTPUT_ACTIVE_LOW, NULL }, {
PWM_OUTPUT_DISABLED, NULL }, { PWM_OUTPUT_DISABLED, NULL } },
/* HW dependent part.*/
0 };

int main(void) {
	halInit();
	chSysInit();

	// this thread would blink one of the LEDs, that would look cool
	chThdCreateStatic(blinkingThreadStack, sizeof(blinkingThreadStack), NORMALPRIO, blinkingThread, NULL);

	// serial-over-usb initialization
	usb_serial_start();

	pwmStart(PWM_SLOW, &pwmcfg_slow);
	palSetPadMode(GPIOB, 7, PAL_MODE_ALTERNATE(2));
	pwmEnableChannel(PWM_SLOW, 1, 600);

	while (TRUE)
		chThdSleep(100);

	return 0;
}

